@@ -288,19 +288,21 @@ tp_chat_sender_ready_notify_cb (EmpathyContact *contact,
 	EmpathyTpChatPriv   *priv = GET_PRIV (chat);
 	EmpathyMessage      *message;
 	EmpathyContactReady  ready;
-	EmpathyContact      *sender;
+	EmpathyContact      *sender = NULL;
 	gboolean             removed = FALSE;
 
 	/* Emit all messages queued until we find a message with not
-	 * ready sender. When leaving this loop, sender is the first not ready
-	 * contact queued and removed tells if at least one message got removed
+	 * ready sender (in case of a MUC we could have more than one sender).
+	 * When leaving this loop, sender is the first not ready contact queued
+	 * and removed tells if at least one message got removed
 	 * from the queue. */
 	while (priv->message_queue) {
 		message = priv->message_queue->data;
 		sender = empathy_message_get_sender (message);
 		ready = empathy_contact_get_ready (sender);
 
-		if (!(ready & EMPATHY_CONTACT_READY_NAME)) {
+		if ((ready & EMPATHY_CONTACT_READY_NAME) == 0 ||
+		    (ready & EMPATHY_CONTACT_READY_ID) == 0) {
 			break;
 		}
 
@@ -313,11 +315,15 @@ tp_chat_sender_ready_notify_cb (EmpathyContact *contact,
 	}
 
 	if (removed) {
+		/* We removed at least one message from the queue, disconnect
+		 * the ready signal from the previous contact */
 		g_signal_handlers_disconnect_by_func (contact,
 						      tp_chat_sender_ready_notify_cb,
 						      chat);
 
 		if (priv->message_queue) {
+			/* We still have queued message, connect the ready
+			 * signal on the new first message sender. */
 			g_signal_connect (sender, "notify::ready",
 					  G_CALLBACK (tp_chat_sender_ready_notify_cb),
 					  chat);
@@ -342,7 +348,8 @@ tp_chat_emit_or_queue_message (EmpathyTpChat  *chat,
 
 	sender = empathy_message_get_sender (message);
 	ready = empathy_contact_get_ready (sender);
-	if (ready & EMPATHY_CONTACT_READY_NAME) {
+	if ((ready & EMPATHY_CONTACT_READY_NAME) &&
+	    (ready & EMPATHY_CONTACT_READY_ID)) {
 		DEBUG ("Message queue empty and sender ready");
 		g_signal_emit (chat, signals[MESSAGE_RECEIVED], 0, message);
 		return;
@@ -446,6 +453,21 @@ tp_chat_send_error_cb (TpChannel   *channel,
 	g_object_unref (message);
 }
 
+static void
+tp_chat_send_cb (TpChannel    *proxy,
+		 const GError *error,
+		 gpointer      user_data,
+		 GObject      *chat)
+{
+	EmpathyMessage *message = EMPATHY_MESSAGE (user_data);
+
+	if (error) {
+		DEBUG ("Error: %s", error->message);
+		g_signal_emit (chat, signals[SEND_ERROR], 0, message,
+			       TP_CHANNEL_TEXT_SEND_ERROR_UNKNOWN);
+	}
+}
+
 static void
 tp_chat_state_changed_cb (TpChannel *channel,
 			  guint      handle,
@@ -1205,8 +1227,9 @@ empathy_tp_chat_send (EmpathyTpChat *chat,
 	tp_cli_channel_type_text_call_send (priv->channel, -1,
 					    message_type,
 					    message_body,
-					    tp_chat_async_cb,
-					    "sending message", NULL,
+					    tp_chat_send_cb,
+					    g_object_ref (message),
+					    (GDestroyNotify) g_object_unref,
 					    G_OBJECT (chat));
 }
 
