@@ -33,15 +33,13 @@
 #include "empathy-tp-contact-list.h"
 #include "empathy-contact-list.h"
 #include "empathy-tp-group.h"
-#include "empathy-debug.h"
 #include "empathy-utils.h"
 
-#define GET_PRIV(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), \
-		       EMPATHY_TYPE_TP_CONTACT_LIST, EmpathyTpContactListPriv))
+#define DEBUG_FLAG EMPATHY_DEBUG_TP | EMPATHY_DEBUG_CONTACT
+#include "empathy-debug.h"
 
-#define DEBUG_DOMAIN "TpContactList"
-
-struct _EmpathyTpContactListPriv {
+#define GET_PRIV(obj) EMPATHY_GET_PRIV (obj, EmpathyTpContactList)
+typedef struct {
 	McAccount      *account;
 	TpConnection   *connection;
 	const gchar    *protocol_group;
@@ -54,7 +52,7 @@ struct _EmpathyTpContactListPriv {
 
 	GList          *groups;
 	GHashTable     *contacts_groups;
-};
+} EmpathyTpContactListPriv;
 
 typedef enum {
 	TP_CONTACT_LIST_TYPE_PUBLISH,
@@ -62,8 +60,6 @@ typedef enum {
 	TP_CONTACT_LIST_TYPE_UNKNOWN
 } TpContactListType;
 
-static void empathy_tp_contact_list_class_init (EmpathyTpContactListClass *klass);
-static void empathy_tp_contact_list_init       (EmpathyTpContactList      *list);
 static void tp_contact_list_iface_init         (EmpathyContactListIface   *iface);
 
 enum {
@@ -88,8 +84,7 @@ tp_contact_list_group_destroy_cb (EmpathyTpGroup       *group,
 {
 	EmpathyTpContactListPriv *priv = GET_PRIV (list);
 
-	empathy_debug (DEBUG_DOMAIN, "Group destroyed: %s",
-		       empathy_tp_group_get_name (group));
+	DEBUG ("Group destroyed: %s", empathy_tp_group_get_name (group));
 
 	priv->groups = g_list_remove (priv->groups, group);
 	g_object_unref (group);
@@ -121,10 +116,10 @@ tp_contact_list_group_member_added_cb (EmpathyTpGroup       *group,
 
 	group_name = empathy_tp_group_get_name (group);
 	if (!g_list_find_custom (*groups, group_name, (GCompareFunc) strcmp)) {
-		empathy_debug (DEBUG_DOMAIN, "Contact %s (%d) added to group %s",
-			       empathy_contact_get_id (contact),
-			       empathy_contact_get_handle (contact),
-			       group_name);
+		DEBUG ("Contact %s (%d) added to group %s",
+			empathy_contact_get_id (contact),
+			empathy_contact_get_handle (contact),
+			group_name);
 		*groups = g_list_prepend (*groups, g_strdup (group_name));
 		g_signal_emit_by_name (list, "groups-changed", contact,
 				       group_name,
@@ -155,10 +150,10 @@ tp_contact_list_group_member_removed_cb (EmpathyTpGroup       *group,
 
 	group_name = empathy_tp_group_get_name (group);
 	if ((l = g_list_find_custom (*groups, group_name, (GCompareFunc) strcmp))) {
-		empathy_debug (DEBUG_DOMAIN, "Contact %s (%d) removed from group %s",
-			       empathy_contact_get_id (contact),
-			       empathy_contact_get_handle (contact),
-			       group_name);
+		DEBUG ("Contact %s (%d) removed from group %s",
+			empathy_contact_get_id (contact),
+			empathy_contact_get_handle (contact),
+			group_name);
 		*groups = g_list_delete_link (*groups, l);
 		g_signal_emit_by_name (list, "groups-changed", contact,
 				       group_name,
@@ -240,10 +235,10 @@ tp_contact_list_added_cb (EmpathyTpGroup       *group,
 	TpContactListType         list_type;
 
 	list_type = tp_contact_list_get_type (list, group);
-	empathy_debug (DEBUG_DOMAIN, "Contact %s (%d) added to list type %d",
-		      empathy_contact_get_id (contact),
-		      empathy_contact_get_handle (contact),
-		      list_type);
+	DEBUG ("Contact %s (%d) added to list type %d",
+		empathy_contact_get_id (contact),
+		empathy_contact_get_handle (contact),
+		list_type);
 
 	/* We now get the presence of that contact, add it to members */
 	if (list_type == TP_CONTACT_LIST_TYPE_SUBSCRIBE &&
@@ -274,10 +269,10 @@ tp_contact_list_removed_cb (EmpathyTpGroup       *group,
 	TpContactListType         list_type;
 
 	list_type = tp_contact_list_get_type (list, group);
-	empathy_debug (DEBUG_DOMAIN, "Contact %s (%d) removed from list type %d",
-		      empathy_contact_get_id (contact),
-		      empathy_contact_get_handle (contact),
-		      list_type);
+	DEBUG ("Contact %s (%d) removed from list type %d",
+		empathy_contact_get_id (contact),
+		empathy_contact_get_handle (contact),
+		list_type);
 
 	/* This contact refuses to send us his presence, remove from members. */
 	if (list_type == TP_CONTACT_LIST_TYPE_SUBSCRIBE &&
@@ -312,10 +307,10 @@ tp_contact_list_pending_cb (EmpathyTpGroup       *group,
 	TpContactListType         list_type;
 
 	list_type = tp_contact_list_get_type (list, group);
-	empathy_debug (DEBUG_DOMAIN, "Contact %s (%d) pending in list type %d",
-		      empathy_contact_get_id (contact),
-		      empathy_contact_get_handle (contact),
-		      list_type);
+	DEBUG ("Contact %s (%d) pending in list type %d",
+		empathy_contact_get_id (contact),
+		empathy_contact_get_handle (contact),
+		list_type);
 
 	/* We want this contact in our contact list but we don't get its 
 	 * presence yet. Add to members anyway. */
@@ -350,7 +345,7 @@ tp_contact_list_invalidated_cb (TpConnection         *connection,
 	EmpathyTpContactListPriv *priv = GET_PRIV (list);
 	GList                    *l;
 
-	empathy_debug (DEBUG_DOMAIN, "Connection invalidated");
+	DEBUG ("Connection invalidated");
 
 	/* Remove all contacts */
 	for (l = priv->members; l; l = l->next) {
@@ -390,12 +385,13 @@ tp_contact_list_add_channel (EmpathyTpContactList *list,
 			     guint                 handle)
 {
 	EmpathyTpContactListPriv *priv = GET_PRIV (list);
-	EmpathyTpGroup           *group;
 	TpChannel                *channel;
+	EmpathyTpGroup           *group;
+	const gchar              *group_name;
+	GList                    *contacts, *l;
 
 	if (strcmp (channel_type, TP_IFACE_CHANNEL_TYPE_CONTACT_LIST) != 0 ||
-	    (handle_type != TP_HANDLE_TYPE_LIST &&
-	     handle_type != TP_HANDLE_TYPE_GROUP)) {
+	    handle_type != TP_HANDLE_TYPE_GROUP) {
 		return;
 	}
 
@@ -407,122 +403,35 @@ tp_contact_list_add_channel (EmpathyTpContactList *list,
 	empathy_run_until_ready (group);
 	g_object_unref (channel);
 
-	if (handle_type == TP_HANDLE_TYPE_LIST) {
-		TpContactListType  list_type;
-		GList             *contacts, *l;
-
-		list_type = tp_contact_list_get_type (list, group);
-		if (list_type == TP_CONTACT_LIST_TYPE_PUBLISH && !priv->publish) {
-			priv->publish = g_object_ref (group);
-
-			/* Publish is the list of contacts to who we send our
-			 * presence. Makes no sense to be in remote-pending */
-			g_signal_connect (group, "local-pending",
-					  G_CALLBACK (tp_contact_list_pending_cb),
-					  list);
-
-			contacts = empathy_tp_group_get_local_pendings (group);
-			for (l = contacts; l; l = l->next) {
-				EmpathyPendingInfo *info = l->data;
-
-				tp_contact_list_pending_cb (group,
-							    info->member,
-							    info->actor,
-							    0,
-							    info->message,
-							    list);
-				empathy_pending_info_free (info);
-			}
-			g_list_free (contacts);
-		}
-		else if (list_type == TP_CONTACT_LIST_TYPE_SUBSCRIBE && !priv->subscribe) {
-			priv->subscribe = g_object_ref (group);
-
-			/* Subscribe is the list of contacts from who we
-			 * receive presence. Makes no sense to be in
-			 * local-pending */
-			g_signal_connect (group, "remote-pending",
-					  G_CALLBACK (tp_contact_list_pending_cb),
-					  list);
-
-			contacts = empathy_tp_group_get_remote_pendings (group);
-			for (l = contacts; l; l = l->next) {
-				tp_contact_list_pending_cb (group,
-							    l->data,
-							    NULL, 0,
-							    NULL, list);
-				g_object_unref (l->data);
-			}
-			g_list_free (contacts);
-		} else {
-			empathy_debug (DEBUG_DOMAIN,
-				      "Type of contact list channel unknown "
-				      "or aleady have that list: %s",
-				      empathy_tp_group_get_name (group));
-			goto OUT;
-		}
-		empathy_debug (DEBUG_DOMAIN,
-			       "New contact list channel of type: %d",
-			       list_type);
-
-		g_signal_connect (group, "member-added",
-				  G_CALLBACK (tp_contact_list_added_cb),
-				  list);
-		g_signal_connect (group, "member-removed",
-				  G_CALLBACK (tp_contact_list_removed_cb),
-				  list);
-
-		contacts = empathy_tp_group_get_members (group);
-		for (l = contacts; l; l = l->next) {
-			tp_contact_list_added_cb (group,
-						  l->data,
-						  NULL, 0, NULL,
-						  list);
-			g_object_unref (l->data);
-		}
-		g_list_free (contacts);
-	}
-	else if (handle_type == TP_HANDLE_TYPE_GROUP) {
-		const gchar *group_name;
-		GList       *contacts, *l;
-
-		/* Check if already exists */
-		group_name = empathy_tp_group_get_name (group);
-		if (tp_contact_list_find_group (list, group_name)) {
-			goto OUT;
-		}
-
-		empathy_debug (DEBUG_DOMAIN, "New server-side group channel: %s",
-			       group_name);
-
-		priv->groups = g_list_prepend (priv->groups, g_object_ref (group));
-
-		g_signal_connect (group, "member-added",
-				  G_CALLBACK (tp_contact_list_group_member_added_cb),
-				  list);
-		g_signal_connect (group, "member-removed",
-				  G_CALLBACK (tp_contact_list_group_member_removed_cb),
-				  list);
-		g_signal_connect (group, "destroy",
-				  G_CALLBACK (tp_contact_list_group_destroy_cb),
-				  list);
-
-		contacts = empathy_tp_group_get_members (group);
-		for (l = contacts; l; l = l->next) {
-			tp_contact_list_group_member_added_cb (group, l->data,
-							       NULL, 0, NULL,
-							       list);
-			g_object_unref (l->data);
-		}
-		g_list_free (contacts);
-	} else {
-		empathy_debug (DEBUG_DOMAIN,
-			       "Unknown handle type (%d) for contact list channel",
-			       handle_type);
+	/* Check if already exists */
+	group_name = empathy_tp_group_get_name (group);
+	if (tp_contact_list_find_group (list, group_name)) {
+		g_object_unref (group);
+		return;
 	}
 
-OUT:
-	g_object_unref (group);
+	/* Add the group */
+	DEBUG ("New server-side group: %s", group_name);
+	priv->groups = g_list_prepend (priv->groups, group);
+	g_signal_connect (group, "member-added",
+			  G_CALLBACK (tp_contact_list_group_member_added_cb),
+			  list);
+	g_signal_connect (group, "member-removed",
+			  G_CALLBACK (tp_contact_list_group_member_removed_cb),
+			  list);
+	g_signal_connect (group, "destroy",
+			  G_CALLBACK (tp_contact_list_group_destroy_cb),
+			  list);
+
+	/* Get initial members */
+	contacts = empathy_tp_group_get_members (group);
+	for (l = contacts; l; l = l->next) {
+		tp_contact_list_group_member_added_cb (group, l->data,
+						       NULL, 0, NULL,
+						       list);
+		g_object_unref (l->data);
+	}
+	g_list_free (contacts);
 }
 
 static void
@@ -555,9 +464,7 @@ tp_contact_list_list_channels_cb (TpConnection    *connection,
 	guint                     i;
 
 	if (error) {
-		empathy_debug (DEBUG_DOMAIN,
-			      "Failed to get list of open channels: %s",
-			      error ? error->message : "No error given");
+		DEBUG ("Error: %s", error->message);
 		return;
 	}
 
@@ -582,6 +489,145 @@ tp_contact_list_list_channels_cb (TpConnection    *connection,
 	priv->ready = TRUE;
 }
 
+static void
+tp_contact_list_request_channel_cb (TpConnection *connection,
+				    const gchar  *object_path,
+				    const GError *error,
+				    gpointer      user_data,
+				    GObject      *weak_object)
+{
+	EmpathyTpContactList     *list = EMPATHY_TP_CONTACT_LIST (weak_object);
+	EmpathyTpContactListPriv *priv = GET_PRIV (list);
+	EmpathyTpGroup           *group;
+	TpChannel                *channel;
+	TpContactListType         list_type;
+	GList                    *contacts, *l;
+
+	if (error) {
+		DEBUG ("Error: %s", error->message);
+		return;
+	}
+
+	channel = tp_channel_new (connection, object_path,
+				  TP_IFACE_CHANNEL_TYPE_CONTACT_LIST,
+				  TP_HANDLE_TYPE_LIST,
+				  GPOINTER_TO_UINT (user_data),
+				  NULL);
+	group = empathy_tp_group_new (channel);
+	empathy_run_until_ready (group);
+
+	list_type = tp_contact_list_get_type (list, group);
+	if (list_type == TP_CONTACT_LIST_TYPE_PUBLISH && !priv->publish) {
+		DEBUG ("Got publish list");
+		priv->publish = group;
+
+		/* Publish is the list of contacts to who we send our
+		 * presence. Makes no sense to be in remote-pending */
+		g_signal_connect (group, "local-pending",
+				  G_CALLBACK (tp_contact_list_pending_cb),
+				  list);
+
+		contacts = empathy_tp_group_get_local_pendings (group);
+		for (l = contacts; l; l = l->next) {
+			EmpathyPendingInfo *info = l->data;
+				tp_contact_list_pending_cb (group,
+						    info->member,
+						    info->actor,
+						    0,
+						    info->message,
+						    list);
+			empathy_pending_info_free (info);
+		}
+		g_list_free (contacts);
+	}
+	else if (list_type == TP_CONTACT_LIST_TYPE_SUBSCRIBE && !priv->subscribe) {
+		DEBUG ("Got subscribe list");
+		priv->subscribe = group;
+
+		/* Subscribe is the list of contacts from who we
+		 * receive presence. Makes no sense to be in
+		 * local-pending */
+		g_signal_connect (group, "remote-pending",
+				  G_CALLBACK (tp_contact_list_pending_cb),
+				  list);
+
+		contacts = empathy_tp_group_get_remote_pendings (group);
+		for (l = contacts; l; l = l->next) {
+			tp_contact_list_pending_cb (group,
+						    l->data,
+						    NULL, 0,
+						    NULL, list);
+			g_object_unref (l->data);
+		}
+		g_list_free (contacts);
+	} else {
+		DEBUG ("Type of contact list channel unknown or aleady "
+			"have that list: %s",
+			empathy_tp_group_get_name (group));
+		g_object_unref (group);
+		return;
+	}
+
+	/* For all list types when need to get members */
+	g_signal_connect (group, "member-added",
+			  G_CALLBACK (tp_contact_list_added_cb),
+			  list);
+	g_signal_connect (group, "member-removed",
+			  G_CALLBACK (tp_contact_list_removed_cb),
+			  list);
+
+	contacts = empathy_tp_group_get_members (group);
+	for (l = contacts; l; l = l->next) {
+		tp_contact_list_added_cb (group,
+					  l->data,
+					  NULL, 0, NULL,
+					  list);
+		g_object_unref (l->data);
+	}
+	g_list_free (contacts);
+}
+
+static void
+tp_contact_list_request_handle_cb (TpConnection *connection,
+				   const GArray *handles,
+				   const GError *error,
+				   gpointer      user_data,
+				   GObject      *list)
+{
+	guint handle;
+
+	if (error) {
+		DEBUG ("Error: %s", error->message);
+		return;
+	}
+
+	handle = g_array_index (handles, guint, 0);
+	tp_cli_connection_call_request_channel (connection, -1,
+						TP_IFACE_CHANNEL_TYPE_CONTACT_LIST,
+						TP_HANDLE_TYPE_LIST,
+						handle,
+						TRUE,
+						tp_contact_list_request_channel_cb,
+						GUINT_TO_POINTER (handle), NULL,
+						list);
+}
+
+static void
+tp_contact_list_request_list (EmpathyTpContactList *list,
+			      const gchar          *type)
+{
+	EmpathyTpContactListPriv *priv = GET_PRIV (list);
+	const gchar *names[] = {type, NULL};
+
+	tp_cli_connection_call_request_handles (priv->connection,
+						-1,
+						TP_HANDLE_TYPE_LIST,
+						names,
+						tp_contact_list_request_handle_cb,
+						NULL, NULL,
+						G_OBJECT (list));
+}
+
 static void
 tp_contact_list_finalize (GObject *object)
 {
@@ -591,7 +637,7 @@ tp_contact_list_finalize (GObject *object)
 	list = EMPATHY_TP_CONTACT_LIST (object);
 	priv = GET_PRIV (list);
 
-	empathy_debug (DEBUG_DOMAIN, "finalize: %p", object);
+	DEBUG ("finalize: %p", object);
 
 	if (priv->subscribe) {
 		g_object_unref (priv->subscribe);
@@ -621,19 +667,37 @@ tp_contact_list_finalize (GObject *object)
 }
 
 static void
-tp_contact_list_ready_cb (EmpathyTpContactList *list)
+tp_contact_list_connection_ready (TpConnection *connection,
+				  const GError *error,
+				  gpointer      list)
 {
 	EmpathyTpContactListPriv *priv = GET_PRIV (list);
 
+	if (error) {
+		tp_contact_list_invalidated_cb (connection,
+						error->domain,
+						error->code,
+						error->message,
+						EMPATHY_TP_CONTACT_LIST (list));
+		return;
+	}
+
+	g_signal_connect (priv->connection, "invalidated",
+			  G_CALLBACK (tp_contact_list_invalidated_cb),
+			  list);
+
+	tp_contact_list_request_list (list, "publish");
+	tp_contact_list_request_list (list, "subscribe");
+
 	tp_cli_connection_call_list_channels (priv->connection, -1,
 					      tp_contact_list_list_channels_cb,
 					      NULL, NULL,
-					      G_OBJECT (list));
+					      list);
 
 	tp_cli_connection_connect_to_new_channel (priv->connection,
 						  tp_contact_list_new_channel_cb,
 						  NULL, NULL,
-						  G_OBJECT (list), NULL);
+						  list, NULL);
 }
 
 static void
@@ -643,7 +707,6 @@ tp_contact_list_constructed (GObject *list)
 	EmpathyTpContactListPriv *priv = GET_PRIV (list);
 	MissionControl           *mc;
 	guint                     status;
-	gboolean                  ready;
 	McProfile                *profile;
 	const gchar              *protocol_name;
 
@@ -655,17 +718,9 @@ tp_contact_list_constructed (GObject *list)
 	g_return_if_fail (priv->connection != NULL);
 	g_object_unref (mc);
 
-	g_signal_connect (priv->connection, "invalidated",
-			  G_CALLBACK (tp_contact_list_invalidated_cb),
-			  list);
-	g_object_get (priv->connection, "connection-ready", &ready, NULL);
-	if (ready) {
-		tp_contact_list_ready_cb (EMPATHY_TP_CONTACT_LIST (list));
-	} else {
-		g_signal_connect_swapped (priv->connection, "notify::connection-ready",
-					  G_CALLBACK (tp_contact_list_ready_cb),
-					  list);
-	}
+	tp_connection_call_when_ready (priv->connection,
+				       tp_contact_list_connection_ready,
+				       list);
 
 	/* Check for protocols that does not support contact groups. We can
 	 * put all contacts into a special group in that case.
@@ -749,8 +804,10 @@ empathy_tp_contact_list_class_init (EmpathyTpContactListClass *klass)
 static void
 empathy_tp_contact_list_init (EmpathyTpContactList *list)
 {
-	EmpathyTpContactListPriv *priv = GET_PRIV (list);
+	EmpathyTpContactListPriv *priv = G_TYPE_INSTANCE_GET_PRIVATE (list,
+		EMPATHY_TYPE_TP_CONTACT_LIST, EmpathyTpContactListPriv);
 
+	list->priv = priv;
 	priv->contacts_groups = g_hash_table_new_full (g_direct_hash,
 						       g_direct_equal,
 						       (GDestroyNotify) g_object_unref,
@@ -786,8 +843,11 @@ tp_contact_list_add (EmpathyContactList *list,
 
 	g_return_if_fail (EMPATHY_IS_TP_CONTACT_LIST (list));
 
-	empathy_tp_group_add_member (priv->subscribe, contact, message);
-	if (g_list_find (priv->pendings, contact)) {
+	if (priv->subscribe) {
+		empathy_tp_group_add_member (priv->subscribe, contact, message);
+	}
+
+	if (priv->publish && g_list_find (priv->pendings, contact)) {
 		empathy_tp_group_add_member (priv->publish, contact, message);		
 	}
 }
@@ -801,8 +861,12 @@ tp_contact_list_remove (EmpathyContactList *list,
 
 	g_return_if_fail (EMPATHY_IS_TP_CONTACT_LIST (list));
 
-	empathy_tp_group_remove_member (priv->subscribe, contact, message);
-	empathy_tp_group_remove_member (priv->publish, contact, message);		
+	if (priv->subscribe) {
+		empathy_tp_group_remove_member (priv->subscribe, contact, message);
+	}
+	if (priv->publish) {
+		empathy_tp_group_remove_member (priv->publish, contact, message);		
+	}
 }
 
 static GList *
@@ -893,16 +957,15 @@ tp_contact_list_get_group (EmpathyTpContactList *list,
 		return tp_group;
 	}
 
-	empathy_debug (DEBUG_DOMAIN, "creating new group: %s", group);
+	DEBUG ("creating new group: %s", group);
 
 	if (!tp_cli_connection_run_request_handles (priv->connection, -1,
 						    TP_HANDLE_TYPE_GROUP,
 						    names,
 						    &handles,
 						    &error, NULL)) {
-		empathy_debug (DEBUG_DOMAIN,
-			      "Failed to RequestHandles: %s",
-			      error ? error->message : "No error given");
+		DEBUG ("Failed to RequestHandles: %s",
+			error ? error->message : "No error given");
 		g_clear_error (&error);
 		return NULL;
 	}
@@ -916,9 +979,8 @@ tp_contact_list_get_group (EmpathyTpContactList *list,
 						    TRUE,
 						    &object_path,
 						    &error, NULL)) {
-		empathy_debug (DEBUG_DOMAIN,
-			      "Failed to RequestChannel: %s",
-			      error ? error->message : "No error given");
+		DEBUG ("Failed to RequestChannel: %s",
+			error ? error->message : "No error given");
 		g_clear_error (&error);
 		return NULL;
 	}
@@ -981,7 +1043,7 @@ tp_contact_list_rename_group (EmpathyContactList *list,
 		return;
 	}
 
-	empathy_debug (DEBUG_DOMAIN, "rename group %s to %s", old_group, new_group);
+	DEBUG ("rename group %s to %s", old_group, new_group);
 
 	/* Remove all members from the old group */
 	members = empathy_tp_group_get_members (tp_group);
@@ -1013,7 +1075,7 @@ tp_contact_list_remove_group (EmpathyContactList *list,
 		return;
 	}
 
-	empathy_debug (DEBUG_DOMAIN, "remove group %s", group);
+	DEBUG ("remove group %s", group);
 
 	/* Remove all members of the group */
 	members = empathy_tp_group_get_members (tp_group);
