@@ -722,6 +722,7 @@ main_window_favorite_chatroom_join (EmpathyChatroom *chatroom)
 {
 	MissionControl *mc;
 	McAccount      *account;
+	TpConnection   *connection;
 	const gchar    *room;
 
 	mc = empathy_mission_control_new ();
@@ -730,12 +731,14 @@ main_window_favorite_chatroom_join (EmpathyChatroom *chatroom)
 
 	DEBUG ("Requesting channel for '%s'", room);
 
-	mission_control_request_channel_with_string_handle (mc,
-							    account,
-							    TP_IFACE_CHANNEL_TYPE_TEXT,
-							    room,
-							    TP_HANDLE_TYPE_ROOM,
-							    NULL, NULL);	
+	connection = mission_control_get_tpconnection (mc, account, NULL);
+	tp_connection_run_until_ready (connection, TRUE, NULL, NULL);	
+	empathy_connection_request_channel (connection, -1,
+					    TP_IFACE_CHANNEL_TYPE_TEXT,
+					    TP_HANDLE_TYPE_ROOM,
+					    room, TRUE,
+					    NULL, NULL, NULL, NULL);
+	g_object_unref (connection);
 	g_object_unref (mc);
 }
 
